@@ -1,6 +1,4 @@
-# encoding: ASCII-8BIT
 # frozen_string_literal: true
-# TODO: remove encoding comment
 
 require "sexp"
 require "ruby_lexer"
@@ -30,7 +28,7 @@ class Sexp
 end
 
 module RubyParserStuff
-  VERSION = "3.21.1"
+  VERSION = "3.22.0"
 
   attr_accessor :lexer, :in_def, :in_single, :file, :in_argdef
   attr_accessor :in_kwarg
@@ -92,9 +90,6 @@ module RubyParserStuff
     [k, true]
   }.to_h
 
-  # TODO: remove
-  has_enc = "".respond_to? :encoding
-
   # This is in sorted order of occurrence according to
   # charlock_holmes against 500k files, with UTF_8 forced
   # to the top.
@@ -108,7 +103,7 @@ module RubyParserStuff
     Encoding::SHIFT_JIS,
     Encoding::WINDOWS_1252,
     Encoding::EUC_JP
-  ] if has_enc
+  ]
 
   JUMP_TYPE = [:return, :next, :break, :yield].map { |k| [k, true] }.to_h
 
@@ -225,8 +220,8 @@ module RubyParserStuff
   end
 
   def endless_method_name defn_or_defs
-    name = defn_or_defs[1]
-    name = defn_or_defs[2] unless Symbol === name
+    _, name, maybe_name, * = defn_or_defs
+    name = maybe_name unless Symbol === name
 
     if attrset_id? name then
       yyerror "setter method cannot be defined in an endless method definition"
@@ -254,6 +249,36 @@ module RubyParserStuff
   end
 
   def assignable(lhs, value = nil)
+    id, line = lhs.last, lhs.line
+
+    result =
+      case lhs.sexp_type
+      when :const then
+        s(:cdecl, id)
+      when :cvar then
+        asgn = in_def || in_single > 0
+        s((asgn ? :cvasgn : :cvdecl), id)
+      when :gvar then
+        s(:gasgn, id)
+      when :ivar then
+        s(:iasgn, id)
+      else
+        case self.env[id]
+        when :lvar, :dvar, nil then
+          self.env[id] ||= :lvar
+          s(:lasgn, id)
+        else
+          raise "wtf? unknown type: #{self.env[id]}"
+        end
+      end
+
+    result << value if value
+    result.line line
+
+    result
+  end
+
+  def old_assignable(lhs, value = nil)
     id, line = lhs
     id = id.to_sym
 
@@ -339,7 +364,7 @@ module RubyParserStuff
       when Sexp then
         case arg.sexp_type
         when :array, :args, :call_args then # HACK? remove array at some point
-          result.concat arg.sexp_body
+          result.sexp_body += arg.sexp_body
         else
           result << arg
         end
@@ -567,14 +592,13 @@ module RubyParserStuff
 
   def handle_encoding str
     str = str.dup
-    has_enc = str.respond_to? :encoding # TODO: remove
     encoding = nil
 
     header = str.each_line.first(2)
-    header.map! { |s| s.force_encoding "ASCII-8BIT" } if has_enc
+    header.map! { |s| s.force_encoding "ASCII-8BIT" }
 
     first = header.first || ""
-    encoding, str = +"utf-8", str.b[3..-1] if first =~ /\A\xEF\xBB\xBF/
+    encoding, str = +"utf-8", str.b[3..-1] if first =~ /\A\xEF\xBB\xBF/n
 
     encoding = $1.strip if header.find { |s|
       s[/^#.*?-\*-.*?coding:\s*([^ ;]+).*?-\*-/, 1] ||
@@ -582,15 +606,11 @@ module RubyParserStuff
     }
 
     if encoding then
-      if has_enc then
-        encoding.sub!(/utf-8-.+$/, "utf-8") # HACK for stupid emacs formats
-        hack_encoding str, encoding
-      else
-        warn "Skipping magic encoding comment"
-      end
+      encoding.sub!(/utf-8-.+$/, "utf-8") # HACK for stupid emacs formats
+      hack_encoding str, encoding
     else
       # nothing specified... ugh. try to encode as utf-8
-      hack_encoding str if has_enc
+      hack_encoding str
     end
 
     str
@@ -645,13 +665,20 @@ module RubyParserStuff
     when :dstr then
       if htype == :str then
         lineno = head.line
-        tail[1] = head.last + tail[1]
+        _, h1 = head
+        _, t1, *rest = tail
+        tail.sexp_body = [h1 + t1, *rest]
+
         head = tail
         head.line = lineno
       else
         tail.sexp_type = :array
-        tail[1] = s(:str, tail[1]).line tail.line
-        tail.delete_at 1 if tail[1] == s(:str, "")
+        _, tail_s, *tail_r = tail
+        if tail_s == "" then
+          tail.sexp_body = tail_r
+        else
+          tail.sexp_body = [s(:str, tail_s).line(tail.line), *tail_r]
+        end
 
         head.push(*tail.sexp_body)
       end
@@ -663,9 +690,11 @@ module RubyParserStuff
         head.line = l
       end
 
-      if head.size == 2 and tail.size > 1 and tail[1].sexp_type == :str then
-        head[-1] = head[-1].dup if head[-1].frozen?
-        head.last << tail[1].last
+      _, t1, * = tail
+      if head.size == 2 and tail.size > 1 and t1.sexp_type == :str then
+        _, h1 = head
+        head.sexp_body = [h1.dup] if h1.frozen? # this is dumb
+        head.last << t1.last
         head.sexp_type = :str if head.size == 2 # HACK ?
       else
         head.push(tail)
@@ -847,7 +876,7 @@ module RubyParserStuff
 
       while res do
         result << res
-        res = res.resbody(true)
+        res = res.find_node :resbody, :delete
       end
 
       result << elsebody if elsebody
@@ -916,8 +945,9 @@ module RubyParserStuff
       body = body.delete_at 3
     end
 
-    result[2..-1].each do |node|
-      block = node.block(:delete)
+    _, _expr, *cases = result
+    cases.each do |node|
+      block = node.find_node :block, :delete
       node.concat block.sexp_body if block
     end
 
@@ -1129,8 +1159,7 @@ module RubyParserStuff
 
     if kw_rest_arg then
       name = kw_rest_arg.value
-      # TODO: I _hate_ this:
-      assignable [name, kw_rest_arg.line] if name != :**
+      assignable kw_rest_arg if name != :**
       result << kw_rest_arg
     end
 
@@ -1333,23 +1362,25 @@ module RubyParserStuff
 
     case node.sexp_type
     when :str then
+      _, str = node
       node.sexp_type = :lit
-      node[1] = if k then
-                  Regexp.new(node[1], o, k)
-                else
-                  begin
-                    Regexp.new(node[1], o)
-                  rescue RegexpError => e
-                    warn "WARNING: #{e.message} for #{node[1].inspect} #{options.inspect}"
-                    begin
-                      warn "WARNING: trying to recover with ENC_UTF8"
-                      Regexp.new(node[1], Regexp::ENC_UTF8)
-                    rescue RegexpError => e
-                      warn "WARNING: trying to recover with ENC_NONE"
-                      Regexp.new(node[1], Regexp::ENC_NONE)
-                    end
-                  end
+      val = if k then
+              Regexp.new(str, o, k)
+            else
+              begin
+                Regexp.new(str, o)
+              rescue RegexpError => e
+                warn "WARNING: #{e.message} for #{str.inspect} #{options.inspect}"
+                begin
+                  warn "WARNING: trying to recover with ENC_UTF8"
+                  Regexp.new(str, Regexp::ENC_UTF8)
+                rescue RegexpError => e
+                  warn "WARNING: trying to recover with ENC_NONE"
+                  Regexp.new(str, Regexp::ENC_NONE)
                 end
+              end
+            end
+      node.sexp_body = [val]
     when :dstr then
       if options =~ /o/ then
         node.sexp_type = :dregx_once
@@ -1572,7 +1603,7 @@ module RubyParserStuff
 
       self.lexer.string = handle_encoding str
 
-      self.file = file.dup
+      self.file = file
 
       @yydebug = ENV.has_key? "DEBUG"
 
