@@ -176,8 +176,6 @@ class TestRubyLexer < Minitest::Test
 
   def test_pct_w_backslashes
     ["\t", "\n", "\r", "\v", "\f"].each do |char|
-      next if !RubyLexer::HAS_ENC and char == "\v"
-
       assert_lex("%w[foo#{char}bar]",
                  s(:array, s(:str, "foo"), s(:str, "bar")),
 
@@ -584,7 +582,7 @@ class TestRubyLexer < Minitest::Test
                 :tNL,      nil, EXPR_BEG,
                 :tINTEGER, 2,   EXPR_NUM)
 
-    assert_equal "# one\n# two\n", @lex.comments
+    assert_equal "# one\n# two\n", @lex.comment
   end
 
   def test_yylex_comment_begin
@@ -592,13 +590,13 @@ class TestRubyLexer < Minitest::Test
                 nil,
                 :tINTEGER, 42, EXPR_NUM)
 
-    assert_equal "=begin\nblah\nblah\n=end\n", @lex.comments
+    assert_equal "=begin\nblah\nblah\n=end\n", @lex.comment
   end
 
   def test_yylex_comment_begin_bad
     refute_lex("=begin\nblah\nblah\n")
 
-    assert_equal "", @lex.comments
+    assert_nil @lex.comment
   end
 
   def test_yylex_comment_begin_not_comment
@@ -617,13 +615,13 @@ class TestRubyLexer < Minitest::Test
   def test_yylex_comment_begin_space
     assert_lex3("=begin blah\nblah\n=end\n", nil)
 
-    assert_equal "=begin blah\nblah\n=end\n", @lex.comments
+    assert_equal "=begin blah\nblah\n=end\n", @lex.comment
   end
 
   def test_yylex_comment_end_space_and_text
     assert_lex3("=begin blah\nblah\n=end blab\n", nil)
 
-    assert_equal "=begin blah\nblah\n=end blab\n", @lex.comments
+    assert_equal "=begin blah\nblah\n=end blab\n", @lex.comment
   end
 
   def test_yylex_comment_eos
@@ -2174,6 +2172,8 @@ class TestRubyLexer < Minitest::Test
   end
 
   def test_yylex_question_control_escape
+    skip "bug in prism parser. Use --parser=parse.y" if ?\C-\] != "\u001D"
+
     assert_lex3('?\C-\]', nil, :tSTRING, ?\C-\], EXPR_END)
   end
 
